import pytest
from unittest import mock


@pytest.fixture(scope="function")
def hub(hub):
    for dyne in [R__DYNES__]:
        hub.pop.sub.add(dyne_name=dyne)
    yield hub


@pytest.fixture(scope="function")
def mock_hub(hub):
    m_hub = hub.pop.testing.mock_hub()
    m_hub.OPT = mock.MagicMock()
    yield m_hub



import pathlib

TESTS = "tests"
UNIT = "unit"
INTEGRATION = "integration"


def __init__(hub):
    templates = pathlib.Path(__file__).parent / "_templates"
    # Add all the template files to the hub under their base file_name all uppercase under hub.pop_create.tests
    for template in templates.iterdir():
        if template.is_dir():
            continue
        attr = (
            template.stem.upper().replace("-", "_").replace(".", "_").replace(" ", "_")
        )
        contents = template.read_text()
        setattr(hub.pop_create.tests, attr, contents)


def new(hub, name: str, directory: pathlib.Path, **kwargs):
    test_dir = directory / TESTS
    vertical = kwargs.get("vertical", False)
    clean_name = kwargs.get("clean_name", name)
    dynes = kwargs.get("dynes", ())

    hub.pop_create.init.write(
        directory / "requirements" / "tests.in", contents=hub.pop_create.tests.TESTS,
    )
    hub.pop_create.init.write(
        directory / ".pre-commit-config.yaml",
        contents=hub.pop_create.tests.PRE_COMMIT_CONFIG,
    )
    add_dynes = set(kwargs.get("dynes", ()))
    if not vertical:
        add_dynes.add(clean_name)
    hub.pop_create.init.write(
        test_dir / UNIT / "conftest.py",
        contents=hub.pop_create.tests.UNIT_CONFTEST,
        replace={"R__DYNES__": ", ".join(f'"{d}"' for d in add_dynes),},
    )
    hub.pop_create.init.write(
        test_dir / INTEGRATION / "conftest.py",
        contents=hub.pop_create.tests.INTEGRATION_CONFTEST,
        replace={
            "R__NAME__": name,
            "R__CLEAN_NAME__": clean_name,
            "R__DYNES__": ", ".join(f'"{d}"' for d in add_dynes),
        },
    )

    # Vertically app-merged projects won't have an entrypoint for this test
    if not kwargs.get("vertical"):
        hub.pop_create.init.write(
            test_dir / UNIT / clean_name / "test_seed.py",
            contents=hub.pop_create.tests.UNIT,
            replace={"R__CLEAN_NAME__": clean_name},
        )
        hub.pop_create.init.write(
            test_dir / INTEGRATION / clean_name / "test_seed.py",
            contents=hub.pop_create.tests.INTEGRATION,
            replace={"R__CLEAN_NAME__": clean_name},
        )

    for dyne in dynes:
        (test_dir / UNIT / dyne).mkdir(parents=True, exist_ok=True)
        (test_dir / INTEGRATION / dyne).mkdir(parents=True, exist_ok=True)

    # Make sure that all the directories under "test" are considered modules
    test_paths = {x for x in hub.pop_create.WRITES.keys()}
    for d in test_paths:
        if any(p.name == TESTS for p in d.parents):
            hub.pop_create.init.write(d.parent / "__init__.py", contents="")
    hub.pop_create.init.write(test_dir / "__init__.py", contents="")



def __init__(hub):
    # Remember not to start your app in the __init__ function
    # This function should just be used to set up the plugin subsystem
    # Add another function to call from your run.py to start the app
    for dyne in [R__DYNES__]:
        hub.pop.sub.add(dyne_name=dyne)


def cli(hub):
    hub.pop.config.load(["R__CLEAN_NAME__", R__DYNES__], cli="R__CLEAN_NAME__")
    # Your app's options can now be found under hub.OPT.R__NAME__
    print("R__NAME__ works!")



import pytest
from unittest import mock


@pytest.fixture(scope="session")
def hub(hub):
    for dyne in [R__DYNES__]:
        hub.pop.sub.add(dyne_name=dyne)

    with mock.patch("sys.argv", ["R__NAME__"]):
        hub.pop.config.load([R__DYNES__], "R__CLEAN_NAME__", parse_cli=True)

    yield hub



import pathlib


def __init__(hub):
    templates = pathlib.Path(__file__).parent / "_templates"
    # Add all the template files to the hub under their base file_name all uppercase under hub.pop_create.cicd
    for template in templates.iterdir():
        if template.is_dir():
            continue
        attr = (
            template.stem.upper().replace("-", "_").replace(".", "_").replace(" ", "_")
        )
        contents = template.read_text()
        setattr(hub.pop_create.cicd, attr, contents)


def new(hub, name: str, directory: pathlib.Path, **kwargs):
    clean_name = kwargs.get("clean_name", name)
    hub.pop_create.init.write(
        directory / "build.conf", contents=hub.pop_create.cicd.BUILD,
    )
    hub.pop_create.init.write(
        directory / ".coveragerc",
        contents=hub.pop_create.cicd.COVERAGERC,
        replace={"R__CLEAN_NAME__": clean_name},
    )
    hub.pop_create.init.write(
        directory / ".gitlab-ci.yml", contents=hub.pop_create.cicd.GITLAB_CI,
    )
    hub.pop_create.init.write(
        directory / ".pre-commit-config.yaml",
        contents=hub.pop_create.cicd.PRE_COMMIT_CONFIG,
    )
    hub.pop_create.init.write(
        directory / "noxfile.py",
        contents=hub.pop_create.cicd.NOXFILE,
        replace={"R__CLEAN_NAME__": clean_name},
    )
    hub.pop_create.init.write(
        directory / "cicd" / "upload-code-coverage.sh",
        contents=hub.pop_create.cicd.UPLOAD_CODE_COVERAGE,
    )



import pathlib
import subprocess


def __init__(hub):
    templates = pathlib.Path(__file__).parent / "_templates"
    # Add all the template files to the hub under their base file_name all uppercase under hub.pop_create.docs
    for template in templates.iterdir():
        if template.is_dir():
            continue
        attr = (
            template.stem.upper().replace("-", "_").replace(".", "_").replace(" ", "_")
        )
        contents = template.read_text()
        setattr(hub.pop_create.docs, attr, contents)


def new(hub, name: str, directory: pathlib.Path, **kwargs):
    docs_dir = directory / "docs"
    author = subprocess.getoutput("git config --global user.name")
    clean_name = kwargs.get("clean_name", name)

    hub.pop_create.init.write(
        docs_dir / "make.bat",
        contents=hub.pop_create.docs.MAKE,
        replace={"R__NAME__": name},
    )
    hub.pop_create.init.write(
        docs_dir / "Makefile",
        contents=hub.pop_create.docs.MAKEFILE,
        replace={"R__NAME__": name},
    )
    outline_header = f"{name.title()} Docs Outline"
    eqchars = "=" * len(outline_header)
    hub.pop_create.init.write(
        docs_dir / "outline.rst",
        contents=f"{eqchars}\n{outline_header}\n{eqchars}\n\nA description of my project",
    )
    hub.pop_create.init.write(
        docs_dir / "source" / "conf.py",
        contents=hub.pop_create.docs.CONF,
        replace={
            "R__AUTHOR__": author,
            "R__NAME__": name,
            "R__CLEAN_NAME__": clean_name,
        },
    )
    welcome_header = f"Welcome to {name}'s Documentation!"
    eqchars = "=" * len(welcome_header)
    hub.pop_create.init.write(
        docs_dir / "source" / "index.rst",
        contents=hub.pop_create.docs.INDEX,
        replace={
            "R__CLEAN_NAME__": clean_name,
            "R__WELCOME__": f"{welcome_header}\n{eqchars}\n",
        },
    )
    release_header = f"{name} Release 1.0.0"
    eqchars = "=" * len(release_header)
    hub.pop_create.init.write(
        docs_dir / "source" / "releases" / "1.0.0.rst",
        contents=hub.pop_create.docs.FIRST_RELEASE,
        replace={"R__HEADER__": f"{eqchars}\n{release_header}\n{eqchars}"},
    )
    title = clean_name.replace("_", " ").title()
    eqchars = "=" * len(title)
    hub.pop_create.init.write(
        docs_dir / "source" / "topics" / f"{clean_name}.rst",
        contents=f"{title}\n{eqchars}\n\nA description of my project",
    )
    hub.pop_create.init.write(
        docs_dir / "source" / "tutorial" / "quickstart.rst",
        contents=hub.pop_create.docs.QUICKSTART,
        replace={"R__NAME__": name},
    )



import os

CLI_CONFIG = {
    "project_name": {"subcommands": ["_global_"], "options": ["-n", "--name"],},
    "dyne": {"subcommands": ["_global_"], "options": ["-d"]},
    "ignore_empty_warning": {
        "subcommands": ["_global_"],
        "options": ["-i"],
        "action": "store_true",
    },
    "directory": {"options": ["-D"], "subcommands": ["_global_"]},
    "vertical": {
        "subcommands": ["_global_"],
        "action": "store_true",
        "options": ["-tv"],
    },
}
CONFIG = {
    "project_name": {
        "help": "The name of the project that is being created",
        "default": None,
    },
    "vertical": {
        "default": False,
        "help": "Build a vertically app-merged project, it's entrypoint is in another project",
    },
    "ignore_empty_warning": {
        "default": False,
        "help": "Allow the script to run in a non-empty directory, potentially overwriting files",
    },
    "dyne": {
        "default": [],
        "nargs": "*",
        "help": "A space delimited list of additional dynamic names for vertical app-merging",
    },
    "directory": {
        "default": os.getcwd(),
        "help": "The directory to create the project in",
    },
}

SUBCOMMANDS = {
    "seed": {"help": "Seed a traditional pop project"},
    "tests": {"help": "Create the tests for a traditional pop-project"},
    "docs": {"help": "Create the sphynx tooling for this project"},
    "cicd": {"help": "Create the cicd tooling for this project"},
}
DYNE = {
    "pop_create": ["pop_create"],
}



def test_cli(mock_hub, hub):
    mock_hub.R__CLEAN_NAME__.init.cli = hub.R__CLEAN_NAME__.init.cli
    mock_hub.R__CLEAN_NAME__.init.cli()
    mock_hub.pop.config.load.assert_called_once_with(
        ["R__CLEAN_NAME__"], "R__CLEAN_NAME__"
    )



"""
Seed a new project with a directory tree and first files
"""
import pathlib
import subprocess

COMPLETION_TEXT = """
Congratulations! You now have a project set up called {name}!
This project can be executed by calling the run.py script:
    python3 run.py
This project has been set up with pre-commit hooks for code checks and black.
First set up your source control environment with "git init" or "hg init".
Then enable these checks in your git checkout:
    pip install pre-commit
    pre-commit install
To run pre-commit manually, execute:
    pre-commit run --all-files
Please fork the pop-awesome and open a PR listing your new project \u263A
    https://gitlab.com/saltstack/pop/pop-awesome
"""


def __init__(hub):
    templates = pathlib.Path(__file__).parent / "_templates"
    # Add all the template files to the hub under their base file_name all uppercase under hub.pop_create.seed
    for template in templates.iterdir():
        if template.is_dir():
            continue
        attr = (
            template.stem.upper().replace("-", "_").replace(".", "_").replace(" ", "_")
        )
        contents = template.read_text()
        setattr(hub.pop_create.seed, attr, contents)


def new(hub, name: str, directory: pathlib.Path, **kwargs):
    """
    Given the option in hub.opts "project_name" create a directory tree for a
    new pop project
    """
    author = subprocess.getoutput("git config --global user.name")
    author_email = subprocess.getoutput("git config --global user.email")
    clean_name = kwargs.get("clean_name", name)
    vertical = kwargs.get("vertical", False)

    dynes = list(kwargs.get("dynes", ()))
    if not vertical:
        dynes.append(clean_name)

    dynes = sorted(dynes)

    for dyne in dynes:
        (directory / clean_name / dyne / "contracts").mkdir(parents=True, exist_ok=True)

    dyne_str = ",\n".join(" " * 4 + f'"{dyne}": ["{dyne}"]' for dyne in dynes)
    hub.pop_create.init.write(
        directory / clean_name / "conf.py",
        contents=hub.pop_create.seed.CONF,
        replace={'"R__DYNE__"': f"\n{dyne_str}\n"},
    )
    hub.pop_create.init.write(
        directory / clean_name / "version.py", contents=hub.pop_create.seed.VERSION,
    )
    eqchars = "=" * len(name)
    readme_str = f"{eqchars}\n{name.upper()}\n{eqchars}\n"
    hub.pop_create.init.write(
        directory / "README.rst",
        contents=readme_str,
        replace={"R__NAME__": clean_name},
    )
    hub.pop_create.init.write(
        directory / "requirements" / "base.txt", contents=hub.pop_create.seed.BASE
    )
    hub.pop_create.init.write(
        directory / "LICENSE",
        contents=hub.pop_create.seed.LICENSE,
        replace={"R__AUTHOR__": author},
    )
    hub.pop_create.init.write(
        directory / ".pyproject.toml", contents=hub.pop_create.seed.PYPROJECT
    )
    hub.pop_create.init.write(
        directory / ".pre-commit-config.yaml",
        contents=hub.pop_create.seed.PRE_COMMIT_CONFIG,
    )
    hub.pop_create.init.write(
        directory / ".gitignore", contents=hub.pop_create.seed.GITIGNORE
    )

    if kwargs.get("vertical"):
        hub.pop_create.init.write(
            directory / "setup.py",
            contents=hub.pop_create.seed.SETUP,
            replace={
                "R__NAME__": name,
                "R__CLEAN_NAME__": clean_name,
                "R__ENTRY__": "",
                "R__AUTHOR__": author,
                "R__AUTHOR_EMAIL__": author_email,
            },
        )
    else:
        (directory / clean_name / clean_name / "contracts").mkdir(
            parents=True, exist_ok=True
        )
        hub.pop_create.init.write(
            directory / "setup.py",
            contents=hub.pop_create.seed.SETUP,
            replace={
                "R__NAME__": name,
                "R__CLEAN_NAME__": clean_name,
                "R__AUTHOR__": author,
                "R__AUTHOR_EMAIL__": author_email,
                "R__ENTRY__": f"{name} = {clean_name}.scripts:start",
            },
        )
        hub.pop_create.init.write(
            directory / clean_name / "scripts.py",
            contents=hub.pop_create.seed.SCRIPT,
            replace={"R__NAME__": clean_name},
        )
        hub.pop_create.init.write(
            directory / "run.py",
            contents=hub.pop_create.seed.RUN,
            replace={"R__NAME__": clean_name},
        )
        add_dynes = set(kwargs.get("dynes", ()))
        hub.pop_create.init.write(
            directory / clean_name / clean_name / "init.py",
            contents=hub.pop_create.seed.INIT,
            replace={
                "R__NAME__": name,
                "R__CLEAN_NAME__": clean_name,
                "R__DYNES__": ", ".join(f'"{d}"' for d in add_dynes),
            },
        )

    print(COMPLETION_TEXT)



import pathlib
from typing import Dict, List


def __init__(hub):
    hub.pop.sub.load_subdirs(hub.pop_create, recurse=True)
    # Hold onto all the writes that should be done until exist checks have been done
    hub.pop_create.WRITES = {}


def cli(hub):
    hub.pop.config.load(["pop_create"], cli="pop_create")
    directory = pathlib.Path(hub.OPT.pop_create.directory)
    name = hub.OPT.pop_create.project_name or directory.name
    if hub.SUBPARSER:
        subparsers = [hub.SUBPARSER.replace("-", "_")]
    else:
        # No subparser specified, do all the core creators
        subparsers = ["seed", "cicd", "docs", "tests"]
    hub.pop_create.init.run(
        subparsers=subparsers,
        name=name,
        directory=directory,
        vertical=hub.OPT.pop_create.vertical,
        dynes=hub.OPT.pop_create.dyne,
    )
    hub.pop_create.init.flush()


def run(hub, subparsers: List[str], name, directory: pathlib.Path, **kwargs):
    clean_name = name.replace("-", "_").replace(" ", "_")
    for subparser in subparsers:
        hub.pop_create[subparser].init.new(
            name=name, clean_name=clean_name, directory=directory, **kwargs,
        )


def write(
    hub, path: pathlib.Path, contents: str = "", replace: Dict[str, str] = None,
):
    # Perform substitutions
    if replace:
        for key, value in replace.items():
            contents = contents.replace(key, value)

    if not hub.OPT.pop_create.ignore_empty_warning and path.exists():
        raise FileExistsError(f"Target is not empty: {path}")

    hub.pop_create.WRITES[path] = contents


def flush(hub):
    for path, content in hub.pop_create.WRITES.items():
        path.parent.mkdir(exist_ok=True, parents=True)
        path.write_text(content)
    hub.pop_create.WRITES.clear()
